import itertools
import logging
import decimal
import json
from functools import partial
from urllib import request

from .error import JSONRPCError


class RPCClient(object):
    def __init__(self, url, rpcuser, rpcpassword, parse_float=decimal.Decimal):
        self._next_id = itertools.count(1).__next__
        self.url = url
        self.rpcuser = rpcuser
        self.rpcpassword = rpcpassword
        self.parse_float = parse_float
        pw_mgr = request.HTTPPasswordMgrWithDefaultRealm()
        pw_mgr.add_password(None, url, user=rpcuser, passwd=rpcpassword)
        auth_handler = request.HTTPBasicAuthHandler(pw_mgr)
        self._request = request.build_opener(auth_handler)

    def __enter__(self):
        return self

    def __exit__(self, exc_type,exc_val,exc_tb):
        self._request.close()

    def __getattr__(self, name):
        return partial(self.call, name)

    def call(self, method, *params):
        _id = self._next_id()

        logging.debug(
            "calling command {} with params {}".format(method, params))

        req = request.Request(self.url, data=json.dumps({
            "jsonrpc": "2.0", "id": _id, "method": method, "params": params
        }).encode('utf-8'))
        
        with self._request.open(req) as response:
            content = response.read().decode('utf-8')
            logging.debug("got response from daemon: " + content)
            resp = json.loads(content, parse_float=self.parse_float)

            if "error" in resp and resp["error"] is not None:
                raise JSONRPCError(resp["error"]["code"], resp["error"]["message"])

            if "result" not in resp:
                raise JSONRPCError(-343, "missing JSON-RPC result")

            return resp["result"]




import itertools
import logging
import decimal
import json
from functools import partial
import httpx

from .error import JSONRPCError


class RPCClientAsync(object):
    def __init__(self, url, rpcuser, rpcpassword, parse_float=decimal.Decimal):
        self._next_id = itertools.count(1).__next__
        self.url = url
        self.rpcuser = rpcuser
        self.rpcpassword = rpcpassword
        self.parse_float = parse_float

        self._request = httpx.AsyncClient(auth=(rpcuser, rpcpassword))

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._request.aclose()

    async def aclose(self):
        await self._request.aclose()

    def __getattr__(self, name):
        return partial(self.call, name)

    async def call(self, method, *params):
        _id = self._next_id()

        logging.debug(
            "calling command {} with params {}".format(method, params))

        response = self._request.post(
            url=self.url,
            content=json.dumps(
                {
                    "jsonrpc": "2.0",
                    "id": _id,
                    "method": method,
                    "params": params,
                }
            ).encode('utf-8'))
        content = (await response).read().decode('utf-8')
        logging.debug("got response from daemon: " + content)
        resp = json.loads(content, parse_float=self.parse_float)

        if "error" in resp and resp["error"] is not None:
            raise JSONRPCError(resp["error"]["code"], resp["error"]["message"])

        if "result" not in resp:
            raise JSONRPCError(-343, "missing JSON-RPC result")

        return resp["result"]



from pathlib import Path
from setuptools import setup

HERE = Path(__file__).parent
README = (HERE / "README.md").read_text()

def get_requirements(req_file):
    """
    Extract requirements from provided file.
    """
    req_path = Path(req_file)
    requirements = req_path.read_text().split("\n") if req_path.exists() else []
    return requirements

setup(
    name="bitcoin-rpc-client",
    version="0.1.0",
    keywords="bitcoin btc json-rpc rpc client",
    description="Bitcoin RPC Client",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/c0mm4nd/bitcoin-rpc-client",
    author="CommandM",
    author_email="maoxs2@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=["bitcoin_rpc_client"],
    include_package_data=True,
    setup_requires=get_requirements("requirements.txt"),
)
